/* --------------- Include libraries --------------- */
#include <Wire.h>                // Library for I2C communication
#include <LiquidCrystal_I2C.h>   // Library for I2C LCD
#include "DHT.h"                 // Library for DHT11

/* ------------ Define pins, constants ------------- */
#define DHTPIN 2 
#define DHTTYPE DHT11 
int VoltSensor1 = A0;  // voltage sensor 1
int VoltSensor2 = A1;  // voltage sensor 2
int CurrSensor1 = A2;  // current sensor 1
int CurrSensor2 = A3;  // current sensor 2
int FAN = 3;           // Fan control output

/* --------------- Define objects ------------------ */
LiquidCrystal_I2C lcd = LiquidCrystal_I2C(0x27, 20, 4);   // Change to (0x27,16,2) for 16x2 LCD. 0x27 is my lcd adress
DHT sensor(DHTPIN, DHTTYPE); 

/* ---------------- Global variables --------------- */
// for voltage measurement
float R1 = 30000;           // onboard resistor 1 value
float R2 = 7500;            // onboard resistor 2 value
float vin_1 = 0, vin_2 = 0; // sensor input voltages
// for current measurement
float AcsValueF_1 = 0, AcsValueF_2 = 0;  // sensor input current
// for DHT11
float temp;           // temperature value
String fan_status;    // fan status
// for display
int InitialStart = 0;  // indicate that the device is started
String dname = "Sensor Board";
String dversion = "v_1.0";


void setup() {  
  // Begin serial communication at a baud rate of 115200:
  Serial.begin(115200);
    
  // Initiate the lcd
  lcd.init();
  lcd.backlight();
  
  // setup DHT11
  sensor.begin(); 

  // setup pins
  pinMode(VoltSensor1, INPUT);  
  pinMode(VoltSensor2, INPUT); 
  pinMode(CurrSensor1, INPUT);  
  pinMode(CurrSensor2, INPUT); 
  pinMode(FAN, OUTPUT);
  digitalWrite(FAN, LOW);

  Serial.println("Starting Measurements...");
}

void measure_volt() {
  int svalue_1 = 0, svalue_2 = 0;
  float vout_1 = 0, vout_2 = 0;
  
  // read from two sensors
  svalue_1 = analogRead(VoltSensor1); 
  svalue_2 = analogRead(VoltSensor2); 
  
  // convert the readings to voltage
  vout_1 = svalue_1 * 5 / 1024;
  vout_2 = svalue_2 * 5 / 1024;
  
  // calculate the real voltages
  vin_1 = vout_1 / (R2 / (R1 + R2));
  vin_2 = vout_2 / (R2 / (R1 + R2));

  Serial.print("Voltage sensor 1: ");
  Serial.print(vin_1);
  Serial.println("V");
  Serial.print("Voltage sensor 2: ");
  Serial.print(vin_2);
  Serial.println("V");
}

void measure_curr() {
  int AcsValue_1 = 0, AcsValue_2 = 0;
  float SumAcs_1 = 0, SumAcs_2 = 0, AvgAcs_1 = 0, AvgAcs_2 = 0;

  // get 150 readings and sum and average them
  for (int i = 0; i < 150; i++) { 
    AcsValue_1 = analogRead(CurrSensor1);
    AcsValue_2 = analogRead(CurrSensor2);     
    SumAcs_1 = SumAcs_1 + AcsValue_1;
    SumAcs_2 = SumAcs_2 + AcsValue_2;
    delay(3);  // let ADC settle before next sample 3ms
  }
  AvgAcs_1 = SumAcs_1 /150;
  AvgAcs_2 = SumAcs_2 /150;   

/* AvgAcs * (5.0 / 1024.0) is converting the read voltage in 0-5 volts
   2.5 is offset(I assumed that arduino is working on 5v so the viout at no current comes
   out to be 2.5 which is out offset. If your arduino is working on different voltage than 
   you must change the offset according to the input voltage)
   0.185v(185mV) is rise in output voltage when 1A current flows at input */
  AcsValueF_1 = (2.5 - (AvgAcs_1 * 5 / 1024)) / 0.185;
  AcsValueF_2 = (2.5 - (AvgAcs_2 * 5 / 1024)) / 0.185;

  Serial.print("Current sensor 1: ");
  Serial.print(AcsValueF_1);
  Serial.println("A");
  Serial.print("Current sensor 2: ");
  Serial.print(AcsValueF_2);
  Serial.println("A");
  
  delay(50);
}

void control_fan() {
  // Read the temperature from the sensor
  temp = sensor.readTemperature();  
  // Check if the sensor is sending values
  if (isnan(temp)) {
    Serial.println("Failed");
    delay(2000);
    return;
  }   
  Serial.print("Temp: ");
  Serial.print(temp);
  Serial.println(" C");
  
  // control the fan
  if (temp > 30) {
    digitalWrite(FAN, HIGH);
    Serial.println("Fan: ON!"); 
    fan_status = "ON";
    delay(10);
  }
  else {
    digitalWrite(FAN, LOW);    
    Serial.println("Fan: OFF!"); 
    fan_status = "OFF";
  }
  delay(100); 
  lcd.clear();    
}

void display_readings() {  
  // display the device name and version for 3 seconds at the beginning.
  if (InitialStart == 0) {    
    lcd.setCursor(3, 1);    // Set the cursor on col: 4, row: 2
    lcd.print(dname);       // Print device name
    lcd.setCursor(6, 2);    // Set the cursor on col: 7, row: 3
    lcd.print(dversion);    // Print device version
//    lcd.noCursor();
    delay(3000);
    lcd.clear();            // Delete the screen
    InitialStart = 1;
  }    
  // display voltage
  lcd.setCursor(0, 0);  // Set the cursor on col: 1, row: 1
  lcd.print("CH1:");    // print channel
  lcd.setCursor(4, 0);  // Set the cursor on col: 5, row: 1
  lcd.print(vin_1);     // Print voltage on channel 1
  lcd.setCursor(8, 0);  // Set the cursor on col: 9, row: 1
  lcd.print("V");       // Print "V"
  lcd.setCursor(10, 0);  // Set the cursor on col: 11, row: 1
  lcd.print("CH2:");    // print channel
  lcd.setCursor(14, 0);  // Set the cursor on col: 15, row: 1
  lcd.print(vin_2);     // Print voltage on channel 2
  lcd.setCursor(19, 0);  // Set the cursor on col: 20, row: 1
  lcd.print("V");       //  Print "V"
  
  // display current
  lcd.setCursor(0, 1);  // Set the cursor on col: 1, row: 2
  lcd.print("CH1:");    // print channel
  lcd.setCursor(4, 1);  // Set the cursor on col: 5, row: 2
  lcd.print(AcsValueF_1);     // Print voltage on channel 1
  lcd.setCursor(8, 1);  // Set the cursor on col: 9, row: 2
  lcd.print("A");       // Print "V"
  lcd.setCursor(10, 1);  // Set the cursor on col: 11, row: 2
  lcd.print("CH2:");    // print channel
  lcd.setCursor(14, 1);  // Set the cursor on col: 15, row: 2
  lcd.print(AcsValueF_2);     // Print voltage on channel 2
  lcd.setCursor(19, 1);  // Set the cursor on col: 20, row: 2
  lcd.print("A");       //  Print "V"
  
  // display temperature and fan status
  lcd.setCursor(0, 2);  // Set the cursor on col: 1, row: 3
  lcd.print("Temp:");    // print channel
  lcd.setCursor(5, 2);  // Set the cursor on col: 6, row: 3
  lcd.print(temp);     // Print voltage on channel 1
  lcd.setCursor(9, 2);  // Set the cursor on col: 10, row: 3
  lcd.print("C");       // Print "C"
  lcd.setCursor(12, 2);  // Set the cursor on col: 13, row: 3
  lcd.print("FAN:");    // print channel
  lcd.setCursor(16, 2);  // Set the cursor on col: 17, row: 3
  lcd.print(fan_status);     // Print voltage on channel 2  

  // display fault
  
}

void loop() {
    
  measure_volt();      // measure the voltages from two voltage sensors
  measure_curr();      // measure the current from two current sensors
  control_fan();       // control the fan based on DHT sensor value
  display_readings();  // display all readings

  Serial.println("**************************");
  delay(1000);
}
